package javax.sql.rowset;

import java.net.URL;
import java.sql.NClob;
import java.sql.RowId;
import java.sql.SQLXML;
import java.util.Calendar;
import javax.sql.rowset.serial.SerialArray;
import java.sql.Array;
import javax.sql.rowset.serial.SerialClob;
import java.sql.Clob;
import javax.sql.rowset.serial.SerialBlob;
import java.sql.Blob;
import javax.sql.rowset.serial.SerialRef;
import java.sql.Ref;
import java.sql.SQLFeatureNotSupportedException;
import java.sql.Timestamp;
import java.sql.Time;
import java.sql.Date;
import java.math.BigDecimal;
import java.util.Iterator;
import javax.sql.RowSetEvent;
import java.sql.SQLException;
import javax.sql.RowSet;
import java.util.Hashtable;
import javax.sql.RowSetListener;
import java.util.Vector;
import java.util.Map;
import java.io.Reader;
import java.io.InputStream;
import java.io.Serializable;

public abstract class BaseRowSet implements Serializable, Cloneable
{
    public static final int UNICODE_STREAM_PARAM = 0;
    public static final int BINARY_STREAM_PARAM = 1;
    public static final int ASCII_STREAM_PARAM = 2;
    protected InputStream binaryStream;
    protected InputStream unicodeStream;
    protected InputStream asciiStream;
    protected Reader charStream;
    private String command;
    private String URL;
    private String dataSource;
    private transient String username;
    private transient String password;
    private int rowSetType;
    private boolean showDeleted;
    private int queryTimeout;
    private int maxRows;
    private int maxFieldSize;
    private int concurrency;
    private boolean readOnly;
    private boolean escapeProcessing;
    private int isolation;
    private int fetchDir;
    private int fetchSize;
    private Map<String, Class<?>> map;
    private Vector<RowSetListener> listeners;
    private Hashtable<Integer, Object> params;
    static final long serialVersionUID = 4886719666485113312L;
    
    public BaseRowSet() {
        this.rowSetType = 1004;
        this.showDeleted = false;
        this.queryTimeout = 0;
        this.maxRows = 0;
        this.maxFieldSize = 0;
        this.concurrency = 1008;
        this.escapeProcessing = true;
        this.fetchDir = 1000;
        this.fetchSize = 0;
        this.listeners = new Vector();
    }
    
    protected void initParams() {
        this.params = new Hashtable();
    }
    
    public void addRowSetListener(RowSetListener e) {
        this.listeners.add(e);
    }
    
    public void removeRowSetListener(RowSetListener o) {
        this.listeners.remove(o);
    }
    
    private void checkforRowSetInterface() throws SQLException {
        if (!(this instanceof RowSet)) {
            throw new SQLException("The class extending abstract class BaseRowSet must implement javax.sql.RowSet or one of it's sub-interfaces.");
        }
        return;
    }
    
    protected void notifyCursorMoved() throws SQLException {
        this.checkforRowSetInterface();
        if (!this.listeners.isEmpty()) {
            RowSetEvent rowSetEvent = new RowSetEvent((RowSet)this);
            Iterator<RowSetListener> iterator = this.listeners.iterator();
            while (iterator.hasNext()) {
                ((RowSetListener)iterator.next()).cursorMoved(rowSetEvent);
            }
        }
    }
    
    protected void notifyRowChanged() throws SQLException {
        this.checkforRowSetInterface();
        if (!this.listeners.isEmpty()) {
            RowSetEvent rowSetEvent = new RowSetEvent((RowSet)this);
            Iterator<RowSetListener> iterator = this.listeners.iterator();
            while (iterator.hasNext()) {
                ((RowSetListener)iterator.next()).rowChanged(rowSetEvent);
            }
        }
    }
    
    protected void notifyRowSetChanged() throws SQLException {
        this.checkforRowSetInterface();
        if (!this.listeners.isEmpty()) {
            RowSetEvent rowSetEvent = new RowSetEvent((RowSet)this);
            Iterator<RowSetListener> iterator = this.listeners.iterator();
            while (iterator.hasNext()) {
                ((RowSetListener)iterator.next()).rowSetChanged(rowSetEvent);
            }
        }
    }
    
    public String getCommand() {
        return this.command;
    }
    
    public void setCommand(String command) throws SQLException {
        if (command == null) {
            this.command = null;
        }
        else {
            if (command.length() == 0) {
                throw new SQLException("Invalid command string detected. Cannot be of length less than 0");
            }
            if (this.params == null) {
                throw new SQLException("Set initParams() before setCommand");
            }
            this.params.clear();
            this.command = command;
        }
    }
    
    public String getUrl() throws SQLException {
        return this.URL;
    }
    
    public void setUrl(String url) throws SQLException {
        if (url != null) {
            if (url.length() < 1) {
                throw new SQLException("Invalid url string detected. Cannot be of length less than 1");
            }
            this.URL = url;
        }
        this.dataSource = null;
    }
    
    public String getDataSourceName() {
        return this.dataSource;
    }
    
    public void setDataSourceName(String dataSource) throws SQLException {
        if (dataSource == null) {
            this.dataSource = null;
        }
        else {
            if (dataSource.equals("")) {
                throw new SQLException("DataSource name cannot be empty string");
            }
            this.dataSource = dataSource;
        }
        this.URL = null;
    }
    
    public String getUsername() {
        return this.username;
    }
    
    public void setUsername(String username) {
        if (username == null) {
            this.username = null;
        }
        else {
            this.username = username;
        }
    }
    
    public String getPassword() {
        return this.password;
    }
    
    public void setPassword(String password) {
        if (password == null) {
            this.password = null;
        }
        else {
            this.password = password;
        }
    }
    
    public void setType(int rowSetType) throws SQLException {
        if (rowSetType != 1003 && rowSetType != 1004 && rowSetType != 1005) {
            throw new SQLException("Invalid type of RowSet set. Must be either ResultSet.TYPE_FORWARD_ONLY or ResultSet.TYPE_SCROLL_INSENSITIVE or ResultSet.TYPE_SCROLL_SENSITIVE.");
        }
        this.rowSetType = rowSetType;
        return;
    }
    
    public int getType() throws SQLException {
        return this.rowSetType;
    }
    
    public void setConcurrency(int concurrency) throws SQLException {
        if (concurrency != 1007 && concurrency != 1008) {
            throw new SQLException("Invalid concurrency set. Must be either ResultSet.CONCUR_READ_ONLY or ResultSet.CONCUR_UPDATABLE.");
        }
        this.concurrency = concurrency;
        return;
    }
    
    public boolean isReadOnly() {
        return this.readOnly;
    }
    
    public void setReadOnly(boolean readOnly) {
        this.readOnly = readOnly;
    }
    
    public int getTransactionIsolation() {
        return this.isolation;
    }
    
    public void setTransactionIsolation(int isolation) throws SQLException {
        if (isolation != 0 && isolation != 2 && isolation != 1 && isolation != 4 && isolation != 8) {
            throw new SQLException("Invalid transaction isolation set. Must be either Connection.TRANSACTION_NONE or Connection.TRANSACTION_READ_UNCOMMITTED or Connection.TRANSACTION_READ_COMMITTED or Connection.RRANSACTION_REPEATABLE_READ or Connection.TRANSACTION_SERIALIZABLE");
        }
        this.isolation = isolation;
        return;
    }
    
    public Map<String, Class<?>> getTypeMap() {
        return this.map;
    }
    
    public void setTypeMap(Map<String, Class<?>> map) {
        this.map = map;
    }
    
    public int getMaxFieldSize() throws SQLException {
        return this.maxFieldSize;
    }
    
    public void setMaxFieldSize(int n) throws SQLException {
        if (n < 0) {
            throw new SQLException("Invalid max field size set. Cannot be of value: " + n);
        }
        this.maxFieldSize = n;
        return;
    }
    
    public int getMaxRows() throws SQLException {
        return this.maxRows;
    }
    
    public void setMaxRows(int n) throws SQLException {
        if (n < 0) {
            throw new SQLException("Invalid max row size set. Cannot be of value: " + n);
        }
        if (n < this.getFetchSize()) {
            throw new SQLException("Invalid max row size set. Cannot be less than the fetchSize.");
        }
        this.maxRows = n;
        return;
    }
    
    public void setEscapeProcessing(boolean escapeProcessing) throws SQLException {
        this.escapeProcessing = escapeProcessing;
    }
    
    public int getQueryTimeout() throws SQLException {
        return this.queryTimeout;
    }
    
    public void setQueryTimeout(int n) throws SQLException {
        if (n < 0) {
            throw new SQLException("Invalid query timeout value set. Cannot be of value: " + n);
        }
        this.queryTimeout = n;
        return;
    }
    
    public boolean getShowDeleted() throws SQLException {
        return this.showDeleted;
    }
    
    public void setShowDeleted(boolean showDeleted) throws SQLException {
        this.showDeleted = showDeleted;
    }
    
    public boolean getEscapeProcessing() throws SQLException {
        return this.escapeProcessing;
    }
    
    public void setFetchDirection(int fetchDir) throws SQLException {
        if ((this.getType() == 1003 && fetchDir != 1000) || (fetchDir != 1000 && fetchDir != 1001 && fetchDir != 1002)) {
            throw new SQLException("Invalid Fetch Direction");
        }
        this.fetchDir = fetchDir;
        return;
    }
    
    public int getFetchDirection() throws SQLException {
        return this.fetchDir;
    }
    
    public void setFetchSize(int fetchSize) throws SQLException {
        if (this.getMaxRows() == 0 && fetchSize >= 0) {
            this.fetchSize = fetchSize;
            return;
        }
        if (fetchSize < 0 || fetchSize > this.getMaxRows()) {
            throw new SQLException("Invalid fetch size set. Cannot be of value: " + fetchSize);
        }
        this.fetchSize = fetchSize;
        return;
    }
    
    public int getFetchSize() throws SQLException {
        return this.fetchSize;
    }
    
    public int getConcurrency() throws SQLException {
        return this.concurrency;
    }
    
    private void checkParamIndex(int n) throws SQLException {
        if (n < 1) {
            throw new SQLException("Invalid Parameter Index");
        }
        return;
    }
    
    public void setNull(int n, int i) throws SQLException {
        this.checkParamIndex(n);
        Object[] value = { null, Integer.valueOf(i) };
        if (this.params == null) {
            throw new SQLException("Set initParams() before setNull");
        }
        this.params.put(Integer.valueOf(n - 1), value);
        return;
    }
    
    public void setNull(int n, int i, String s) throws SQLException {
        this.checkParamIndex(n);
        Object[] value = { null, Integer.valueOf(i), s };
        if (this.params == null) {
            throw new SQLException("Set initParams() before setNull");
        }
        this.params.put(Integer.valueOf(n - 1), value);
        return;
    }
    
    public void setBoolean(int n, boolean b) throws SQLException {
        this.checkParamIndex(n);
        if (this.params == null) {
            throw new SQLException("Set initParams() before setNull");
        }
        this.params.put(Integer.valueOf(n - 1), Boolean.valueOf(b));
        return;
    }
    
    public void setByte(int n, byte b) throws SQLException {
        this.checkParamIndex(n);
        if (this.params == null) {
            throw new SQLException("Set initParams() before setByte");
        }
        this.params.put(Integer.valueOf(n - 1), Byte.valueOf(b));
        return;
    }
    
    public void setShort(int n, short s) throws SQLException {
        this.checkParamIndex(n);
        if (this.params == null) {
            throw new SQLException("Set initParams() before setShort");
        }
        this.params.put(Integer.valueOf(n - 1), Short.valueOf(s));
        return;
    }
    
    public void setInt(int n, int i) throws SQLException {
        this.checkParamIndex(n);
        if (this.params == null) {
            throw new SQLException("Set initParams() before setInt");
        }
        this.params.put(Integer.valueOf(n - 1), Integer.valueOf(i));
        return;
    }
    
    public void setLong(int n, long l) throws SQLException {
        this.checkParamIndex(n);
        if (this.params == null) {
            throw new SQLException("Set initParams() before setLong");
        }
        this.params.put(Integer.valueOf(n - 1), Long.valueOf(l));
        return;
    }
    
    public void setFloat(int n, float f) throws SQLException {
        this.checkParamIndex(n);
        if (this.params == null) {
            throw new SQLException("Set initParams() before setFloat");
        }
        this.params.put(Integer.valueOf(n - 1), Float.valueOf(f));
        return;
    }
    
    public void setDouble(int n, double d) throws SQLException {
        this.checkParamIndex(n);
        if (this.params == null) {
            throw new SQLException("Set initParams() before setDouble");
        }
        this.params.put(Integer.valueOf(n - 1), Double.valueOf(d));
        return;
    }
    
    public void setBigDecimal(int n, BigDecimal value) throws SQLException {
        this.checkParamIndex(n);
        if (this.params == null) {
            throw new SQLException("Set initParams() before setBigDecimal");
        }
        this.params.put(Integer.valueOf(n - 1), value);
        return;
    }
    
    public void setString(int n, String value) throws SQLException {
        this.checkParamIndex(n);
        if (this.params == null) {
            throw new SQLException("Set initParams() before setString");
        }
        this.params.put(Integer.valueOf(n - 1), value);
        return;
    }
    
    public void setBytes(int n, byte[] value) throws SQLException {
        this.checkParamIndex(n);
        if (this.params == null) {
            throw new SQLException("Set initParams() before setBytes");
        }
        this.params.put(Integer.valueOf(n - 1), value);
        return;
    }
    
    public void setDate(int n, Date value) throws SQLException {
        this.checkParamIndex(n);
        if (this.params == null) {
            throw new SQLException("Set initParams() before setDate");
        }
        this.params.put(Integer.valueOf(n - 1), value);
        return;
    }
    
    public void setTime(int n, Time value) throws SQLException {
        this.checkParamIndex(n);
        if (this.params == null) {
            throw new SQLException("Set initParams() before setTime");
        }
        this.params.put(Integer.valueOf(n - 1), value);
        return;
    }
    
    public void setTimestamp(int n, Timestamp value) throws SQLException {
        this.checkParamIndex(n);
        if (this.params == null) {
            throw new SQLException("Set initParams() before setTimestamp");
        }
        this.params.put(Integer.valueOf(n - 1), value);
        return;
    }
    
    public void setAsciiStream(int n, InputStream inputStream, int i) throws SQLException {
        this.checkParamIndex(n);
        Object[] value = { inputStream, Integer.valueOf(i), Integer.valueOf(2) };
        if (this.params == null) {
            throw new SQLException("Set initParams() before setAsciiStream");
        }
        this.params.put(Integer.valueOf(n - 1), value);
        return;
    }
    
    public void setAsciiStream(int n, InputStream inputStream) throws SQLException {
        throw new SQLFeatureNotSupportedException("Feature not supported");
    }
    
    public void setBinaryStream(int n, InputStream inputStream, int i) throws SQLException {
        this.checkParamIndex(n);
        Object[] value = { inputStream, Integer.valueOf(i), Integer.valueOf(1) };
        if (this.params == null) {
            throw new SQLException("Set initParams() before setBinaryStream");
        }
        this.params.put(Integer.valueOf(n - 1), value);
        return;
    }
    
    public void setBinaryStream(int n, InputStream inputStream) throws SQLException {
        throw new SQLFeatureNotSupportedException("Feature not supported");
    }
    
    @Deprecated
    public void setUnicodeStream(int n, InputStream inputStream, int i) throws SQLException {
        this.checkParamIndex(n);
        Object[] value = { inputStream, Integer.valueOf(i), Integer.valueOf(0) };
        if (this.params == null) {
            throw new SQLException("Set initParams() before setUnicodeStream");
        }
        this.params.put(Integer.valueOf(n - 1), value);
        return;
    }
    
    public void setCharacterStream(int n, Reader reader, int i) throws SQLException {
        this.checkParamIndex(n);
        Object[] value = { reader, Integer.valueOf(i) };
        if (this.params == null) {
            throw new SQLException("Set initParams() before setCharacterStream");
        }
        this.params.put(Integer.valueOf(n - 1), value);
        return;
    }
    
    public void setCharacterStream(int n, Reader reader) throws SQLException {
        throw new SQLFeatureNotSupportedException("Feature not supported");
    }
    
    public void setObject(int n, Object o, int i, int j) throws SQLException {
        this.checkParamIndex(n);
        Object[] value = { o, Integer.valueOf(i), Integer.valueOf(j) };
        if (this.params == null) {
            throw new SQLException("Set initParams() before setObject");
        }
        this.params.put(Integer.valueOf(n - 1), value);
        return;
    }
    
    public void setObject(int n, Object o, int i) throws SQLException {
        this.checkParamIndex(n);
        Object[] value = { o, Integer.valueOf(i) };
        if (this.params == null) {
            throw new SQLException("Set initParams() before setObject");
        }
        this.params.put(Integer.valueOf(n - 1), value);
        return;
    }
    
    public void setObject(int n, Object value) throws SQLException {
        this.checkParamIndex(n);
        if (this.params == null) {
            throw new SQLException("Set initParams() before setObject");
        }
        this.params.put(Integer.valueOf(n - 1), value);
        return;
    }
    
    public void setRef(int n, Ref ref) throws SQLException {
        this.checkParamIndex(n);
        if (this.params == null) {
            throw new SQLException("Set initParams() before setRef");
        }
        this.params.put(Integer.valueOf(n - 1), new SerialRef(ref));
        return;
    }
    
    public void setBlob(int n, Blob blob) throws SQLException {
        this.checkParamIndex(n);
        if (this.params == null) {
            throw new SQLException("Set initParams() before setBlob");
        }
        this.params.put(Integer.valueOf(n - 1), new SerialBlob(blob));
        return;
    }
    
    public void setClob(int n, Clob clob) throws SQLException {
        this.checkParamIndex(n);
        if (this.params == null) {
            throw new SQLException("Set initParams() before setClob");
        }
        this.params.put(Integer.valueOf(n - 1), new SerialClob(clob));
        return;
    }
    
    public void setArray(int n, Array array) throws SQLException {
        this.checkParamIndex(n);
        if (this.params == null) {
            throw new SQLException("Set initParams() before setArray");
        }
        this.params.put(Integer.valueOf(n - 1), new SerialArray(array));
        return;
    }
    
    public void setDate(int n, Date date, Calendar calendar) throws SQLException {
        this.checkParamIndex(n);
        Object[] value = { date, calendar };
        if (this.params == null) {
            throw new SQLException("Set initParams() before setDate");
        }
        this.params.put(Integer.valueOf(n - 1), value);
        return;
    }
    
    public void setTime(int n, Time time, Calendar calendar) throws SQLException {
        this.checkParamIndex(n);
        Object[] value = { time, calendar };
        if (this.params == null) {
            throw new SQLException("Set initParams() before setTime");
        }
        this.params.put(Integer.valueOf(n - 1), value);
        return;
    }
    
    public void setTimestamp(int n, Timestamp timestamp, Calendar calendar) throws SQLException {
        this.checkParamIndex(n);
        Object[] value = { timestamp, calendar };
        if (this.params == null) {
            throw new SQLException("Set initParams() before setTimestamp");
        }
        this.params.put(Integer.valueOf(n - 1), value);
        return;
    }
    
    public void clearParameters() throws SQLException {
        this.params.clear();
    }
    
    public Object[] getParams() throws SQLException {
        if (this.params == null) {
            this.initParams();
            return new Object[this.params.size()];
        }
        Object[] array = new Object[this.params.size()];
        int i = 0;
        while (i < this.params.size()) {
            array[i] = this.params.get(Integer.valueOf(i));
            if (array[i] == null) {
                throw new SQLException("missing parameter: " + (i + 1));
            }
            ++i;
        }
        return array;
    }
    
    public void setNull(String s, int n) throws SQLException {
        throw new SQLFeatureNotSupportedException("Feature not supported");
    }
    
    public void setNull(String s, int n, String s2) throws SQLException {
        throw new SQLFeatureNotSupportedException("Feature not supported");
    }
    
    public void setBoolean(String s, boolean b) throws SQLException {
        throw new SQLFeatureNotSupportedException("Feature not supported");
    }
    
    public void setByte(String s, byte b) throws SQLException {
        throw new SQLFeatureNotSupportedException("Feature not supported");
    }
    
    public void setShort(String s, short n) throws SQLException {
        throw new SQLFeatureNotSupportedException("Feature not supported");
    }
    
    public void setInt(String s, int n) throws SQLException {
        throw new SQLFeatureNotSupportedException("Feature not supported");
    }
    
    public void setLong(String s, long n) throws SQLException {
        throw new SQLFeatureNotSupportedException("Feature not supported");
    }
    
    public void setFloat(String s, float n) throws SQLException {
        throw new SQLFeatureNotSupportedException("Feature not supported");
    }
    
    public void setDouble(String s, double n) throws SQLException {
        throw new SQLFeatureNotSupportedException("Feature not supported");
    }
    
    public void setBigDecimal(String s, BigDecimal bigDecimal) throws SQLException {
        throw new SQLFeatureNotSupportedException("Feature not supported");
    }
    
    public void setString(String s, String s2) throws SQLException {
        throw new SQLFeatureNotSupportedException("Feature not supported");
    }
    
    public void setBytes(String s, byte[] array) throws SQLException {
        throw new SQLFeatureNotSupportedException("Feature not supported");
    }
    
    public void setTimestamp(String s, Timestamp timestamp) throws SQLException {
        throw new SQLFeatureNotSupportedException("Feature not supported");
    }
    
    public void setAsciiStream(String s, InputStream inputStream, int n) throws SQLException {
        throw new SQLFeatureNotSupportedException("Feature not supported");
    }
    
    public void setBinaryStream(String s, InputStream inputStream, int n) throws SQLException {
        throw new SQLFeatureNotSupportedException("Feature not supported");
    }
    
    public void setCharacterStream(String s, Reader reader, int n) throws SQLException {
        throw new SQLFeatureNotSupportedException("Feature not supported");
    }
    
    public void setAsciiStream(String s, InputStream inputStream) throws SQLException {
        throw new SQLFeatureNotSupportedException("Feature not supported");
    }
    
    public void setBinaryStream(String s, InputStream inputStream) throws SQLException {
        throw new SQLFeatureNotSupportedException("Feature not supported");
    }
    
    public void setCharacterStream(String s, Reader reader) throws SQLException {
        throw new SQLFeatureNotSupportedException("Feature not supported");
    }
    
    public void setNCharacterStream(int n, Reader reader) throws SQLException {
        throw new SQLFeatureNotSupportedException("Feature not supported");
    }
    
    public void setObject(String s, Object o, int n, int n2) throws SQLException {
        throw new SQLFeatureNotSupportedException("Feature not supported");
    }
    
    public void setObject(String s, Object o, int n) throws SQLException {
        throw new SQLFeatureNotSupportedException("Feature not supported");
    }
    
    public void setObject(String s, Object o) throws SQLException {
        throw new SQLFeatureNotSupportedException("Feature not supported");
    }
    
    public void setBlob(int n, InputStream inputStream, long n2) throws SQLException {
        throw new SQLFeatureNotSupportedException("Feature not supported");
    }
    
    public void setBlob(int n, InputStream inputStream) throws SQLException {
        throw new SQLFeatureNotSupportedException("Feature not supported");
    }
    
    public void setBlob(String s, InputStream inputStream, long n) throws SQLException {
        throw new SQLFeatureNotSupportedException("Feature not supported");
    }
    
    public void setBlob(String s, Blob blob) throws SQLException {
        throw new SQLFeatureNotSupportedException("Feature not supported");
    }
    
    public void setBlob(String s, InputStream inputStream) throws SQLException {
        throw new SQLFeatureNotSupportedException("Feature not supported");
    }
    
    public void setClob(int n, Reader reader, long n2) throws SQLException {
        throw new SQLFeatureNotSupportedException("Feature not supported");
    }
    
    public void setClob(int n, Reader reader) throws SQLException {
        throw new SQLFeatureNotSupportedException("Feature not supported");
    }
    
    public void setClob(String s, Reader reader, long n) throws SQLException {
        throw new SQLFeatureNotSupportedException("Feature not supported");
    }
    
    public void setClob(String s, Clob clob) throws SQLException {
        throw new SQLFeatureNotSupportedException("Feature not supported");
    }
    
    public void setClob(String s, Reader reader) throws SQLException {
        throw new SQLFeatureNotSupportedException("Feature not supported");
    }
    
    public void setDate(String s, Date date) throws SQLException {
        throw new SQLFeatureNotSupportedException("Feature not supported");
    }
    
    public void setDate(String s, Date date, Calendar calendar) throws SQLException {
        throw new SQLFeatureNotSupportedException("Feature not supported");
    }
    
    public void setTime(String s, Time time) throws SQLException {
        throw new SQLFeatureNotSupportedException("Feature not supported");
    }
    
    public void setTime(String s, Time time, Calendar calendar) throws SQLException {
        throw new SQLFeatureNotSupportedException("Feature not supported");
    }
    
    public void setTimestamp(String s, Timestamp timestamp, Calendar calendar) throws SQLException {
        throw new SQLFeatureNotSupportedException("Feature not supported");
    }
    
    public void setSQLXML(int n, SQLXML sqlxml) throws SQLException {
        throw new SQLFeatureNotSupportedException("Feature not supported");
    }
    
    public void setSQLXML(String s, SQLXML sqlxml) throws SQLException {
        throw new SQLFeatureNotSupportedException("Feature not supported");
    }
    
    public void setRowId(int n, RowId rowId) throws SQLException {
        throw new SQLFeatureNotSupportedException("Feature not supported");
    }
    
    public void setRowId(String s, RowId rowId) throws SQLException {
        throw new SQLFeatureNotSupportedException("Feature not supported");
    }
    
    public void setNString(int n, String s) throws SQLException {
        throw new SQLFeatureNotSupportedException("Feature not supported");
    }
    
    public void setNString(String s, String s2) throws SQLException {
        throw new SQLFeatureNotSupportedException("Feature not supported");
    }
    
    public void setNCharacterStream(int n, Reader reader, long n2) throws SQLException {
        throw new SQLFeatureNotSupportedException("Feature not supported");
    }
    
    public void setNCharacterStream(String s, Reader reader, long n) throws SQLException {
        throw new SQLFeatureNotSupportedException("Feature not supported");
    }
    
    public void setNCharacterStream(String s, Reader reader) throws SQLException {
        throw new SQLFeatureNotSupportedException("Feature not supported");
    }
    
    public void setNClob(String s, NClob nClob) throws SQLException {
        throw new SQLFeatureNotSupportedException("Feature not supported");
    }
    
    public void setNClob(String s, Reader reader, long n) throws SQLException {
        throw new SQLFeatureNotSupportedException("Feature not supported");
    }
    
    public void setNClob(String s, Reader reader) throws SQLException {
        throw new SQLFeatureNotSupportedException("Feature not supported");
    }
    
    public void setNClob(int n, Reader reader, long n2) throws SQLException {
        throw new SQLFeatureNotSupportedException("Feature not supported");
    }
    
    public void setNClob(int n, NClob nClob) throws SQLException {
        throw new SQLFeatureNotSupportedException("Feature not supported");
    }
    
    public void setNClob(int n, Reader reader) throws SQLException {
        throw new SQLFeatureNotSupportedException("Feature not supported");
    }
    
    public void setURL(int n, URL url) throws SQLException {
        throw new SQLFeatureNotSupportedException("Feature not supported");
    }
}
